#!/usr/bin/env bash

##
## VARIABLES
##

CURDIR=$(dirname $(cd $(dirname $0) && pwd))
WORK_DIR=${WAARP_DIR:-$CURDIR}
INST=$1
LOGFILE="data/$INST/log/waarp-pull.log"
LOCKFILE="data/$INST/log/waarp-pull.lock"
STATUSFILE="data/$INST/log/waarp-pull.status"
LIST_FILE=etc/conf.d/$INST/get-files.list
CURRENT_TRANSFERS=( $(cat $STATUSFILE) )

##
## Functions
##

usage() {
    echo <<EOT
    Usage: $0 HOSTID

Ce script peut être exécuté pour récupérer des fichiers distants disponibles 
sur un serveur distant.

L'interrogation des serveurs distants se fait en utilisant le protocole R66, 
avec le client de l'instance HOSTID.

Les fichiers du serveur distant doivent être situés dans un dossier de la forme

  {outdir}/HOSTID/{identifiant du flux}

où {outdir} est le dossier de départ du serveur distant et {identifiant du flux}
est un identifiant arbitraire pour le flux.

Les dossiers et serveurs interrogés sont listés dans le fichier 
etc/conf.d/HOSTID/get-files.list, au format CSV, dans lequel chaque ligne est 
de la forme :

  identifiant du partenaire,identifiant du flux,règle de transfert
EOT
}

log() {
    echo "[$(date +'%Y-%m-%d %H:%M:%S.%N')] [$$] $@" >> $LOGFILE
}

lock() {
    touch $LOCKFILE
}

unlock() {
    rm -f $LOCKFILE
}

is_locked() {
    [ -f $LOCKFILE ]
}

is_server() {
    [ -f etc/conf.d/$INST/server.xml ]
}

list_files() {
    log "Vérification des fichiers sur $2 dans le dossier $3"
    ./bin/waarp-r66client.sh $INST getinfo -rule $1 -to $2 -file $3 -list -property | grep "^file." | cut -d= -f2
}

get_files() {
    local action=msend
    if is_server; then 
        action=masend
    fi
    ./bin/waarp-r66client.sh $INST $action -rule $1 -to $2 -file $3 -client
}

transfer_exists() {
    [[ " ${CURRENT_TRANSFERS[*]} " == *" $1 "* ]]
}

##
## MAIN
##

cd $WORK_DIR

log "Début du processus"

if is_locked; then 
    log "Un autre processus est en cours"
    exit 0
fi
lock
trap unlock EXIT

if [ -z "$INST" ]; then
    usage
    exit 1
fi
if [ ! -d etc/conf.d/$INST ]; then
    echo "ERREUR: L'instance $INST n'existe pas"
    exit 2
fi

if [ ! -f $LIST_FILE ]; then
    log "Aucun fichier de configuration $LIST_FILE trouvé"
    exit 0
fi


>$STATUSFILE
cat $LIST_FILE | while read -r line
do
    partner=$(echo $line | cut -d, -f1)
    flowid=$(echo $line | cut -d, -f2)
    rule=$(echo $line | cut -d, -f3)
    
    file_list=( $(list_files $rule $partner "$INST/$flowid") )
    log "${#file_list[@]} fichiers disponibles"

    for file in "${file_list[@]}"; do
        filepath="$INST/$flowid/$file"
        if transfer_exists $filepath; then
            log "Un transfert existe déjà pour $filepath"
        else
            log "Création du transfert pour $filepath"
            get_files $rule $partner $filepath
        fi
        echo $filepath >> $STATUSFILE
    done
done

log "Fin du processus"
